dnl ---------------------------------------------------------------------------
dnl
dnl  bonds.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007,2008,2018 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')dnl
include(`scan.m4')dnl
GENERATE_HEADER(bonds.c,
                manipulations of transient bonds.)

#include "common.h"
#include "bonds.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

GENERATE_HEADER_FOR_SCAN

/*---------------------------------------------------------------------------*/
PROTOTYPE_FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`create_bonds_equilibrium',dnl
`transient_bonds *bonds,rng_state *rng,double xi')

PROTOTYPE_FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`construct_bonds_by_transition',dnl
`transient_bonds *bonds,rng_state *rng,double xi,double dt')

PROTOTYPE_FOR_ALL_BONDS(dnl
`destroy_bonds_by_transition',dnl
`rng_state *rng,double dt')

/*---------------------------------------------------------------------------*/
void add_bond(transient_bonds *bonds,int i,int j)
{
    int head_new;

    if(bonds->head == INACTIVE_BOND)
    {
        fprintf(stderr,"warning: cannot add a new bond\n");
        return;
    }

    head_new = bonds->s[bonds->head][1];

    /*printf("%d -> %d: %d %d\n",bonds->head,head_new,i,j);*/

    bonds->s[bonds->head][0] = i;
    bonds->s[bonds->head][1] = j;

    bonds->head  = head_new;
}

/*---------------------------------------------------------------------------*/
void remove_bond(transient_bonds *bonds,int k)
{
    bonds->s[k][0] = INACTIVE_BOND;
    bonds->s[k][1] = bonds->head;

    bonds->head = k;
}

/*---------------------------------------------------------------------------*/
void clear_bonds(transient_bonds *bonds)
{
    int i;

    bonds->head = 0;

    for(i = 0;i < bonds->m_max;i++)
    {
        bonds->s[i][0] = -1;
        bonds->s[i][1] = i + 1;
    }

    bonds->s[bonds->m_max - 1][1] = -1;
}

/*---------------------------------------------------------------------------*/
int total_number_of_bonds(transient_bonds *bonds)
{
    int i,m;

    m = 0;
    for(i = 0;i < bonds->m_max;i++)
    {
        if(IS_ACTIVE_BOND(bonds->s[i]))
            m++;
    }

    return m;
}

/*---------------------------------------------------------------------------*/
void create_bonds_equilibrium(bonded_particles *particles,
                              linked_cell_list *celllist,
                              box_geometry *geometry,
                              rng_state *rng,double xi)
{
    construct_linked_cell_list(celllist,particles,geometry);

    SWITCH_FUNCTION(particles->dimension,
                    create_bonds_equilibrium@D@(particles->particles@D@.r,
                                                celllist,geometry,
                                                &(particles->bonds),rng,xi);)
}

/*---------------------------------------------------------------------------*/
void reconstruct_bonds_by_transition(simulation_condition *condition,
                                     bonded_particles *particles,
                                     linked_cell_list *celllist,
                                     box_geometry *geometry,
                                     rng_state *rng)
{
    SWITCH_FUNCTION(particles->dimension,
                    destroy_bonds_by_transition@D@(particles->particles@D@.r,
                                                     &(particles->bonds),geometry,
                                                     rng,
                                                     (condition->dt / condition->tau));
                    construct_bonds_by_transition@D@(particles->particles@D@.r,
                                                   celllist,geometry,
                                                   &(particles->bonds),rng,
                                                   (condition->xi),
                                                   (condition->dt / condition->tau));)
}

/*---------------------------------------------------------------------------*/
FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`create_bonds_equilibrium',dnl
`transient_bonds *bonds,rng_state *rng,double xi',dnl
`bonds,rng,xi',dnl
`dnl
    double m_bar;
    int l,m;',dnl
`dnl
    m_bar = xi * exp(-0.5 * @D@ * dr2);
    m = prand(rng,m_bar);
    for(l = 0;l < m;l++)
        add_bond(bonds,i,j);')

FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`construct_bonds_by_transition',dnl
`transient_bonds *bonds,rng_state *rng,double xi,double dt',dnl
`bonds,rng,xi,dt',dnl
`double p_plus;',dnl
`dnl
    p_plus = - expm1(-dt * xi * exp(-0.5 * @D@ * dr2));
    if(urand(rng) < p_plus)
        add_bond(bonds,i,j);')

FOR_ALL_BONDS(dnl
`destroy_bonds_by_transition',dnl
`rng_state *rng,double dt',dnl
`double p_minus;',dnl
`dnl
    p_minus = - expm1(-dt);
    if(urand(rng) < p_minus)
        remove_bond(bonds,k);')

/*---------------------------------------------------------------------------*/
