/*---------------------------------------------------------------------------*/

/*!  \file  bonds.h
     \brief  manipulations of transient bonds.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_BONDS_H
#define INCLUDE_BONDS_H

#include "rng.h"
#include "vector.h"
#include "tensor.h"
#include "particles.h"
#include "celllist.h"

/*---------------------------------------------------------------------------*/
/*   \brief  clear all the active transient bond.
     \param  bonds  transient bonds  */
void clear_bonds(transient_bonds *bonds);
/*   \brief  create transient bonds by the equilibrium distribution.
     \param  particles  bonded particles
     \param  celllist  linked cell list
     \param  geometry  box geometry
     \param  rng  rng state
     \param  xi  bond fugacity  */
void create_bonds_equilibrium(bonded_particles *particles,
                              linked_cell_list *celllist,
                              box_geometry *geometry,
                              rng_state *rng,
                              double xi);
/*   \brief  reconstruct transient bonds by the transition dynamics
     \param  condition  simulation condition
     \param  particles  bonded particles
     \param  celllist  linked cell list
     \param  geometry  box geometry
     \param  rng  rng state  */
void reconstruct_bonds_by_transition(simulation_condition *condition,
                                     bonded_particles *particles,
                                     linked_cell_list *celllist,
                                     box_geometry *geometry,
                                     rng_state *rng);
/*   \brief  add a new active transient bond.
     \param  bonds  transient bonds
     \param  i  the first index for the bonded pair
     \param  j  the second index for the bonded pair  */
void add_bond(transient_bonds *bonds,int i,int j);
/*   \brief  remove an active transient bond.
     \param  bonds  transient bonds
     \param  k  the index of a bond to be removed  */
void remove_bond(transient_bonds *bonds,int k);
/*   \brief  get total number of active bonds.
     \param  bonds  transient bonds  */
int total_number_of_bonds(transient_bonds *bonds);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
