/*---------------------------------------------------------------------------*/

/*!  \file  condition.h
     \brief  simulation conditions.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_CONDITION_H
#define INCLUDE_CONDITION_H

/*---------------------------------------------------------------------------*/
/*!  \brief  simulation condition.  */
typedef struct simulation_condition
{
        /*!  \brief  whether to save the data sequantially.  */
    int save_position_sequential;
    int save_bond_sequential;
    int save_stress_sequential;
    int save_energy_sequential;
    int save_geometry_sequential;
    int save_dx_sequential;

        /*!  \brief  seed for a random number generator.  */
    unsigned long seed;

        /*!  \brief  current number of iteration steps.  */
    int iteration_step;

        /*!  \brief  whether to apply the simple shear to the system.  */
    int apply_shear;

        /*!  \brief  maximum nubmer of iteration.  */
    int iteration_max;
        /*!  \brief  interval for writing data.  */
    int interval;

        /*!  \brief  number of particles in the box.  */
    int n;
        /*!  \brief  size of the time step.  */
    double dt;
        /*!  \brief  bond fugacity.  */
    double xi;
        /*!  \brief  bond life time.  */
    double tau;
        /*!  \brief  shear rate.  */
    double kappa;
        /*!  \brief  shear frequency (for oscillatory shear).  */
    double omega;
} simulation_condition;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new simulation condition.  */
simulation_condition *allocate_simulation_condition(void);
/*!  \brief  free a simulation condition.
     \param  condition  simulation condition to free  */
void free_simulation_condition(simulation_condition *condition);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
