/*---------------------------------------------------------------------------*/

/*!  \file  files.c
     \brief  input / output files.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "files.h"

/*---------------------------------------------------------------------------*/
static void free_if_not_null(char *s);

/*---------------------------------------------------------------------------*/
data_files *allocate_data_files(void)
{
    data_files *files;

    files = xmalloc(sizeof(data_files));

    files->position_output = NULL;
    files->bond_output = NULL;
    files->stress_output = NULL;
    files->energy_output = NULL;
    files->geometry_output = NULL;
    files->dx_output = NULL;
    files->h5_output = NULL;

    files->position_template = NULL;
    files->bond_template = NULL;
    files->dx_template = NULL;

    return files;
}

/*---------------------------------------------------------------------------*/
void free_data_files(data_files *files)
{
    free_if_not_null(files->position_output);
    free_if_not_null(files->bond_output);
    free_if_not_null(files->stress_output);
    free_if_not_null(files->energy_output);
    free_if_not_null(files->geometry_output);
    free_if_not_null(files->dx_output);
    free_if_not_null(files->h5_output);

    free_if_not_null(files->position_template);
    free_if_not_null(files->bond_template);
    free_if_not_null(files->dx_template);

    free(files);
}

/*---------------------------------------------------------------------------*/
void free_if_not_null(char *s)
{
    if(s != NULL)
        free(s);
}

/*---------------------------------------------------------------------------*/
