/*---------------------------------------------------------------------------*/

/*!  \file  force.c
     \brief  calculate the random and interaction forces.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from force.c.m4.  */

#include "common.h"
#include "force.h"

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "vector.h"
#include "particles.h"
#include "celllist.h"

#define SQR(x) ((x)*(x))

#define INDEX(i,n) (((i) == -1) ? ((n) - 1) : (((i) == (n)) ? (0) : (i)))
#define LATTICE_VECTOR(i,n,c) (((i) == -1) ? (c) : (((i) == (n)) ? (-(c)) : (0)))
#define IS_LOWER_BOUNDARY(i1,i2,n) (((i1) == 0) && ((i2) == -1))
#define IS_UPPER_BOUNDARY(i1,i2,n) (((i1) == ((n) - 1)) && ((i2) == (n)))


/*---------------------------------------------------------------------------*/
static void generate_random_force1(bonded_particles1 *particles,rng_state *rng);
static void clear_interaction_force1(bonded_particles1 *particles);

static void generate_random_force2(bonded_particles2 *particles,rng_state *rng);
static void clear_interaction_force2(bonded_particles2 *particles);

static void generate_random_force3(bonded_particles3 *particles,rng_state *rng);
static void clear_interaction_force3(bonded_particles3 *particles);

static void accumulate_bond_stress(bonded_particles *particles,
                                   box_geometry *geometry);
static void accumulate_site_stress(bonded_particles *particles,
                                   linked_cell_list *celllist,
                                   box_geometry *geometry,double xi);
static void accumulate_bond_energy(bonded_particles *particles,
                                   box_geometry *geometry);
static void accumulate_site_energy(bonded_particles *particles,
                                   linked_cell_list *celllist,
                                   box_geometry *geometry,double xi);

static void accumulate_bond_force1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           vector1 *f);

static void accumulate_bond_force2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           vector2 *f);

static void accumulate_bond_force3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           vector3 *f);
static void accumulate_bond_stress1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           symmetric_tensor1 sigma);

static void accumulate_bond_stress2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           symmetric_tensor2 sigma);

static void accumulate_bond_stress3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           symmetric_tensor3 sigma);
static void accumulate_bond_energy1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           double *e);

static void accumulate_bond_energy2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           double *e);

static void accumulate_bond_energy3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           double *e);

static void accumulate_site_force1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
                  vector1 *f,double xi);
static void accumulate_site_force1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                               vector1 *f,double xi);
static void accumulate_site_force1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector1 c,
                                     vector1 *f,double xi);

static void accumulate_site_force2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
                  vector2 *f,double xi);
static void accumulate_site_force2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                               vector2 *f,double xi);
static void accumulate_site_force2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector2 c,
                                     vector2 *f,double xi);

static void accumulate_site_force3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
                  vector3 *f,double xi);
static void accumulate_site_force3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                               vector3 *f,double xi);
static void accumulate_site_force3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector3 c,
                                     vector3 *f,double xi);
static void accumulate_site_stress1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
                  symmetric_tensor1 sigma,double xi);
static void accumulate_site_stress1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                               symmetric_tensor1 sigma,double xi);
static void accumulate_site_stress1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector1 c,
                                     symmetric_tensor1 sigma,double xi);

static void accumulate_site_stress2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
                  symmetric_tensor2 sigma,double xi);
static void accumulate_site_stress2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                               symmetric_tensor2 sigma,double xi);
static void accumulate_site_stress2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector2 c,
                                     symmetric_tensor2 sigma,double xi);

static void accumulate_site_stress3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
                  symmetric_tensor3 sigma,double xi);
static void accumulate_site_stress3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                               symmetric_tensor3 sigma,double xi);
static void accumulate_site_stress3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector3 c,
                                     symmetric_tensor3 sigma,double xi);
static void accumulate_site_energy1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
                  double *e,double xi);
static void accumulate_site_energy1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                               double *e,double xi);
static void accumulate_site_energy1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector1 c,
                                     double *e,double xi);

static void accumulate_site_energy2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
                  double *e,double xi);
static void accumulate_site_energy2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                               double *e,double xi);
static void accumulate_site_energy2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector2 c,
                                     double *e,double xi);

static void accumulate_site_energy3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
                  double *e,double xi);
static void accumulate_site_energy3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                               double *e,double xi);
static void accumulate_site_energy3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                                     int k1,int k2,vector3 c,
                                     double *e,double xi);

/*---------------------------------------------------------------------------*/
void generate_random_force(bonded_particles *particles,rng_state *rng)
{
        switch(particles->dimension)
    {
        case 1:
            generate_random_force1(&(particles->particles1),rng);
            break;

        case 2:
            generate_random_force2(&(particles->particles2),rng);
            break;

        case 3:
            generate_random_force3(&(particles->particles3),rng);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void clear_interaction_force(bonded_particles *particles)
{
        switch(particles->dimension)
    {
        case 1:
            clear_interaction_force1(&(particles->particles1));
            break;

        case 2:
            clear_interaction_force2(&(particles->particles2));
            break;

        case 3:
            clear_interaction_force3(&(particles->particles3));
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void clear_stress_tensor(bonded_particles *particles)
{
        switch(particles->dimension)
    {
        case 1:
            particles->particles1.sigma[0] = 0.0;
            break;

        case 2:
            particles->particles2.sigma[0] = 0.0;
particles->particles2.sigma[1] = 0.0;
particles->particles2.sigma[2] = 0.0;
            break;

        case 3:
            particles->particles3.sigma[0] = 0.0;
particles->particles3.sigma[1] = 0.0;
particles->particles3.sigma[2] = 0.0;
particles->particles3.sigma[3] = 0.0;
particles->particles3.sigma[4] = 0.0;
particles->particles3.sigma[5] = 0.0;
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_force(bonded_particles *particles,
                           box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 1:
            accumulate_bond_force1(particles->particles1.r,
                    &(particles->bonds),geometry,
                    particles->particles1.f);
            break;

        case 2:
            accumulate_bond_force2(particles->particles2.r,
                    &(particles->bonds),geometry,
                    particles->particles2.f);
            break;

        case 3:
            accumulate_bond_force3(particles->particles3.r,
                    &(particles->bonds),geometry,
                    particles->particles3.f);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_force_star(bonded_particles *particles,
                                box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 1:
            accumulate_bond_force1(particles->particles1.r_star,
                    &(particles->bonds),geometry,
                    particles->particles1.f_star);
            break;

        case 2:
            accumulate_bond_force2(particles->particles2.r_star,
                    &(particles->bonds),geometry,
                    particles->particles2.f_star);
            break;

        case 3:
            accumulate_bond_force3(particles->particles3.r_star,
                    &(particles->bonds),geometry,
                    particles->particles3.f_star);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_force(bonded_particles *particles,
                           linked_cell_list *celllist,
                           box_geometry *geometry,double xi)
{
        switch(particles->dimension)
    {
        case 1:
            accumulate_site_force1(particles->particles1.r,
                                             celllist,geometry,
                                             particles->particles1.f,xi);
            break;

        case 2:
            accumulate_site_force2(particles->particles2.r,
                                             celllist,geometry,
                                             particles->particles2.f,xi);
            break;

        case 3:
            accumulate_site_force3(particles->particles3.r,
                                             celllist,geometry,
                                             particles->particles3.f,xi);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_force_star(bonded_particles *particles,
                                linked_cell_list *celllist,
                                box_geometry *geometry,double xi)
{
        switch(particles->dimension)
    {
        case 1:
            accumulate_site_force1(particles->particles1.r_star,
                                             celllist,geometry,
                                             particles->particles1.f_star,xi);
            break;

        case 2:
            accumulate_site_force2(particles->particles2.r_star,
                                             celllist,geometry,
                                             particles->particles2.f_star,xi);
            break;

        case 3:
            accumulate_site_force3(particles->particles3.r_star,
                                             celllist,geometry,
                                             particles->particles3.f_star,xi);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_stress(bonded_particles *particles,
                            box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 1:
            accumulate_bond_stress1(particles->particles1.r,
                    &(particles->bonds),geometry,
                    particles->particles1.sigma);
            break;

        case 2:
            accumulate_bond_stress2(particles->particles2.r,
                    &(particles->bonds),geometry,
                    particles->particles2.sigma);
            break;

        case 3:
            accumulate_bond_stress3(particles->particles3.r,
                    &(particles->bonds),geometry,
                    particles->particles3.sigma);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_stress(bonded_particles *particles,
                            linked_cell_list *celllist,
                            box_geometry *geometry,double xi)
{
        switch(particles->dimension)
    {
        case 1:
            accumulate_site_stress1(particles->particles1.r,
                                              celllist,geometry,
                                              particles->particles1.sigma,xi);
            break;

        case 2:
            accumulate_site_stress2(particles->particles2.r,
                                              celllist,geometry,
                                              particles->particles2.sigma,xi);
            break;

        case 3:
            accumulate_site_stress3(particles->particles3.r,
                                              celllist,geometry,
                                              particles->particles3.sigma,xi);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_energy(bonded_particles *particles,
                            box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 1:
            accumulate_bond_energy1(particles->particles1.r,
                    &(particles->bonds),geometry,
                    &(particles->particles1.e[0]));
            break;

        case 2:
            accumulate_bond_energy2(particles->particles2.r,
                    &(particles->bonds),geometry,
                    &(particles->particles2.e[0]));
            break;

        case 3:
            accumulate_bond_energy3(particles->particles3.r,
                    &(particles->bonds),geometry,
                    &(particles->particles3.e[0]));
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_energy(bonded_particles *particles,
                            linked_cell_list *celllist,
                            box_geometry *geometry,double xi)
{
        switch(particles->dimension)
    {
        case 1:
            accumulate_site_energy1(particles->particles1.r,
                                              celllist,geometry,
                                              &(particles->particles1.e[1]),xi);
            break;

        case 2:
            accumulate_site_energy2(particles->particles2.r,
                                              celllist,geometry,
                                              &(particles->particles2.e[1]),xi);
            break;

        case 3:
            accumulate_site_energy3(particles->particles3.r,
                                              celllist,geometry,
                                              &(particles->particles3.e[1]),xi);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void calculate_interaction_force_without_noise(bonded_particles *particles,
                                               linked_cell_list *celllist,
                                               box_geometry *geometry,
                                               double xi)
{
    clear_interaction_force(particles);

    construct_linked_cell_list(celllist,particles,geometry);

    accumulate_site_force(particles,celllist,geometry,xi);
    accumulate_bond_force(particles,geometry);
}

/*---------------------------------------------------------------------------*/
void calculate_stress_tensor(bonded_particles *particles,
                             linked_cell_list *celllist,
                             box_geometry *geometry,double xi)
{
    clear_stress_tensor(particles);

    construct_linked_cell_list(celllist,particles,geometry);

    accumulate_site_stress(particles,celllist,geometry,xi);
    accumulate_bond_stress(particles,geometry);
}

/*---------------------------------------------------------------------------*/
void calculate_potential_energy(bonded_particles *particles,
                                linked_cell_list *celllist,
                                box_geometry *geometry,double xi)
{
        switch(particles->dimension)
    {
        case 1:
            particles->particles1.e[0] = 0.0;
                    particles->particles1.e[1] = 0.0;
            break;

        case 2:
            particles->particles2.e[0] = 0.0;
                    particles->particles2.e[1] = 0.0;
            break;

        case 3:
            particles->particles3.e[0] = 0.0;
                    particles->particles3.e[1] = 0.0;
            break;

        default:
            break;
    }

    construct_linked_cell_list(celllist,particles,geometry);

    accumulate_site_energy(particles,celllist,geometry,xi);
    accumulate_bond_energy(particles,geometry);
}

/*---------------------------------------------------------------------------*/
void generate_random_force1(bonded_particles1 *particles,rng_state *rng)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        particles->w[i][0] = nrand(rng);
    }
}

/*---------------------------------------------------------------------------*/
void clear_interaction_force1(bonded_particles1 *particles)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        particles->f[i][0] = 0.0;
        particles->f_star[i][0] = 0.0;
    }
}

/*---------------------------------------------------------------------------*/
void generate_random_force2(bonded_particles2 *particles,rng_state *rng)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        particles->w[i][0] = nrand(rng);
particles->w[i][1] = nrand(rng);
    }
}

/*---------------------------------------------------------------------------*/
void clear_interaction_force2(bonded_particles2 *particles)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        particles->f[i][0] = 0.0;
particles->f[i][1] = 0.0;
        particles->f_star[i][0] = 0.0;
particles->f_star[i][1] = 0.0;
    }
}

/*---------------------------------------------------------------------------*/
void generate_random_force3(bonded_particles3 *particles,rng_state *rng)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        particles->w[i][0] = nrand(rng);
particles->w[i][1] = nrand(rng);
particles->w[i][2] = nrand(rng);
    }
}

/*---------------------------------------------------------------------------*/
void clear_interaction_force3(bonded_particles3 *particles)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        particles->f[i][0] = 0.0;
particles->f[i][1] = 0.0;
particles->f[i][2] = 0.0;
        particles->f_star[i][0] = 0.0;
particles->f_star[i][1] = 0.0;
particles->f_star[i][2] = 0.0;
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_force1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
           vector1 *f,double xi)
{
    int i1,i2,i2_i;
    int cell_index1,cell_index2;
    vector1 c;
    int i2_min,i2_max;

    for(i1 = 0;i1 < celllist->nx;i1++)
    {
        cell_index1 = i1;

        for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
        {
            i2_i = INDEX(i2,celllist->nx);
            cell_index2 = i2_i;
            c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);

            if(cell_index1 == cell_index2)
            {
                accumulate_site_force1_in_same_cell(r,celllist,cell_index1,
                                  f,xi);
            }
            else if(cell_index1 < cell_index2)
            {
                accumulate_site_force1_in_different_cells(r,celllist,cell_index1,
                                         cell_index2,c,
                                         f,xi);
            }
        }
    }

}

/*---------------------------------------------------------------------------*/
void accumulate_site_force1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                        vector1 *f,double xi)
{
    int i,j;
    double dr2;
    vector1 dr;
    

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];

                dr2 = SQR(dr[0]);
                if(dr2 < celllist->rc2)
                {
                        f[i][0] += xi * exp(-0.5 * 1 * dr2) * 1 * dr[0];
              f[j][0] -= xi * exp(-0.5 * 1 * dr2) * 1 * dr[0];
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_force1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                              int k1,int k2,vector1 c,
                              vector1 *f,double xi)
{
    int i,j;
    double dr2;
    vector1 dr;
    

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];

            dr2 = SQR(dr[0]);
            if(dr2 < celllist->rc2)
            {
                    f[i][0] += xi * exp(-0.5 * 1 * dr2) * 1 * dr[0];
              f[j][0] -= xi * exp(-0.5 * 1 * dr2) * 1 * dr[0];
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void accumulate_site_force2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
           vector2 *f,double xi)
{
    int i1,j1,i2,j2,i2_i,j2_i;
    int cell_index1,cell_index2;
    vector2 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_force2_in_same_cell(r,celllist,cell_index1,
                                               f,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_force2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     f,xi);
                        }
                    }
                }
                

            }
        }
        
    }
    else
    {
            /*  box deformation  */
        

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_force2_in_same_cell(r,celllist,cell_index1,
                                               f,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_force2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     f,xi);
                        }
                    }
                }
                
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_force2_in_same_cell(r,celllist,cell_index1,
                                               f,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_force2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     f,xi);
                        }
                    }
                }
                

            }
        }
        
    }

}

/*---------------------------------------------------------------------------*/
void accumulate_site_force2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                        vector2 *f,double xi)
{
    int i,j;
    double dr2;
    vector2 dr;
    

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

                dr2 = SQR(dr[0]) + SQR(dr[1]);
                if(dr2 < celllist->rc2)
                {
                        f[i][0] += xi * exp(-0.5 * 2 * dr2) * 2 * dr[0];
              f[j][0] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[0];
f[i][1] += xi * exp(-0.5 * 2 * dr2) * 2 * dr[1];
              f[j][1] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[1];
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_force2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                              int k1,int k2,vector2 c,
                              vector2 *f,double xi)
{
    int i,j;
    double dr2;
    vector2 dr;
    

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];

            dr2 = SQR(dr[0]) + SQR(dr[1]);
            if(dr2 < celllist->rc2)
            {
                    f[i][0] += xi * exp(-0.5 * 2 * dr2) * 2 * dr[0];
              f[j][0] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[0];
f[i][1] += xi * exp(-0.5 * 2 * dr2) * 2 * dr[1];
              f[j][1] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[1];
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void accumulate_site_force3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
           vector3 *f,double xi)
{
    int i1,j1,k1,i2,j2,k2,i2_i,j2_i,k2_i;
    int cell_index1,cell_index2;
    vector3 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_force3_in_same_cell(r,celllist,cell_index1,
                                               f,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_force3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     f,xi);
                        }
                    }
                }
                }

            }
        }
        }
    }
    else
    {
            /*  box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_force3_in_same_cell(r,celllist,cell_index1,
                                               f,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_force3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     f,xi);
                        }
                    }
                }
                }
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_force3_in_same_cell(r,celllist,cell_index1,
                                               f,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_force3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     f,xi);
                        }
                    }
                }
                }

            }
        }
        }
    }

}

/*---------------------------------------------------------------------------*/
void accumulate_site_force3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                        vector3 *f,double xi)
{
    int i,j;
    double dr2;
    vector3 dr;
    

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

                dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
                if(dr2 < celllist->rc2)
                {
                        f[i][0] += xi * exp(-0.5 * 3 * dr2) * 3 * dr[0];
              f[j][0] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[0];
f[i][1] += xi * exp(-0.5 * 3 * dr2) * 3 * dr[1];
              f[j][1] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[1];
f[i][2] += xi * exp(-0.5 * 3 * dr2) * 3 * dr[2];
              f[j][2] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[2];
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_force3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                              int k1,int k2,vector3 c,
                              vector3 *f,double xi)
{
    int i,j;
    double dr2;
    vector3 dr;
    

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];
dr[2] = r[i][2] - r[j][2] + c[2];

            dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
            if(dr2 < celllist->rc2)
            {
                    f[i][0] += xi * exp(-0.5 * 3 * dr2) * 3 * dr[0];
              f[j][0] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[0];
f[i][1] += xi * exp(-0.5 * 3 * dr2) * 3 * dr[1];
              f[j][1] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[1];
f[i][2] += xi * exp(-0.5 * 3 * dr2) * 3 * dr[2];
              f[j][2] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[2];
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


void accumulate_site_stress1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
           symmetric_tensor1 sigma,double xi)
{
    int i1,i2,i2_i;
    int cell_index1,cell_index2;
    vector1 c;
    int i2_min,i2_max;

    for(i1 = 0;i1 < celllist->nx;i1++)
    {
        cell_index1 = i1;

        for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
        {
            i2_i = INDEX(i2,celllist->nx);
            cell_index2 = i2_i;
            c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);

            if(cell_index1 == cell_index2)
            {
                accumulate_site_stress1_in_same_cell(r,celllist,cell_index1,
                                  sigma,xi);
            }
            else if(cell_index1 < cell_index2)
            {
                accumulate_site_stress1_in_different_cells(r,celllist,cell_index1,
                                         cell_index2,c,
                                         sigma,xi);
            }
        }
    }

}

/*---------------------------------------------------------------------------*/
void accumulate_site_stress1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                        symmetric_tensor1 sigma,double xi)
{
    int i,j;
    double dr2;
    vector1 dr;
    

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];

                dr2 = SQR(dr[0]);
                if(dr2 < celllist->rc2)
                {
                        sigma[0] -= xi * exp(-0.5 * 1 * dr2) * 1 * dr[0] * dr[0];
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_stress1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                              int k1,int k2,vector1 c,
                              symmetric_tensor1 sigma,double xi)
{
    int i,j;
    double dr2;
    vector1 dr;
    

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];

            dr2 = SQR(dr[0]);
            if(dr2 < celllist->rc2)
            {
                    sigma[0] -= xi * exp(-0.5 * 1 * dr2) * 1 * dr[0] * dr[0];
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void accumulate_site_stress2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
           symmetric_tensor2 sigma,double xi)
{
    int i1,j1,i2,j2,i2_i,j2_i;
    int cell_index1,cell_index2;
    vector2 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_stress2_in_same_cell(r,celllist,cell_index1,
                                               sigma,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_stress2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     sigma,xi);
                        }
                    }
                }
                

            }
        }
        
    }
    else
    {
            /*  box deformation  */
        

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_stress2_in_same_cell(r,celllist,cell_index1,
                                               sigma,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_stress2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     sigma,xi);
                        }
                    }
                }
                
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_stress2_in_same_cell(r,celllist,cell_index1,
                                               sigma,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_stress2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     sigma,xi);
                        }
                    }
                }
                

            }
        }
        
    }

}

/*---------------------------------------------------------------------------*/
void accumulate_site_stress2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                        symmetric_tensor2 sigma,double xi)
{
    int i,j;
    double dr2;
    vector2 dr;
    

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

                dr2 = SQR(dr[0]) + SQR(dr[1]);
                if(dr2 < celllist->rc2)
                {
                        sigma[0] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[0] * dr[0];
sigma[1] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[1] * dr[1];
sigma[2] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[0] * dr[1];
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_stress2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                              int k1,int k2,vector2 c,
                              symmetric_tensor2 sigma,double xi)
{
    int i,j;
    double dr2;
    vector2 dr;
    

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];

            dr2 = SQR(dr[0]) + SQR(dr[1]);
            if(dr2 < celllist->rc2)
            {
                    sigma[0] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[0] * dr[0];
sigma[1] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[1] * dr[1];
sigma[2] -= xi * exp(-0.5 * 2 * dr2) * 2 * dr[0] * dr[1];
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void accumulate_site_stress3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
           symmetric_tensor3 sigma,double xi)
{
    int i1,j1,k1,i2,j2,k2,i2_i,j2_i,k2_i;
    int cell_index1,cell_index2;
    vector3 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_stress3_in_same_cell(r,celllist,cell_index1,
                                               sigma,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_stress3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     sigma,xi);
                        }
                    }
                }
                }

            }
        }
        }
    }
    else
    {
            /*  box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_stress3_in_same_cell(r,celllist,cell_index1,
                                               sigma,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_stress3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     sigma,xi);
                        }
                    }
                }
                }
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_stress3_in_same_cell(r,celllist,cell_index1,
                                               sigma,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_stress3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     sigma,xi);
                        }
                    }
                }
                }

            }
        }
        }
    }

}

/*---------------------------------------------------------------------------*/
void accumulate_site_stress3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                        symmetric_tensor3 sigma,double xi)
{
    int i,j;
    double dr2;
    vector3 dr;
    

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

                dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
                if(dr2 < celllist->rc2)
                {
                        sigma[0] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[0] * dr[0];
sigma[1] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[1] * dr[1];
sigma[2] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[2] * dr[2];
sigma[3] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[0] * dr[1];
sigma[4] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[1] * dr[2];
sigma[5] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[2] * dr[0];
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_stress3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                              int k1,int k2,vector3 c,
                              symmetric_tensor3 sigma,double xi)
{
    int i,j;
    double dr2;
    vector3 dr;
    

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];
dr[2] = r[i][2] - r[j][2] + c[2];

            dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
            if(dr2 < celllist->rc2)
            {
                    sigma[0] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[0] * dr[0];
sigma[1] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[1] * dr[1];
sigma[2] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[2] * dr[2];
sigma[3] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[0] * dr[1];
sigma[4] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[1] * dr[2];
sigma[5] -= xi * exp(-0.5 * 3 * dr2) * 3 * dr[2] * dr[0];
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


void accumulate_site_energy1(vector1 *r,linked_cell_list *celllist,box_geometry *geometry,
           double *e,double xi)
{
    int i1,i2,i2_i;
    int cell_index1,cell_index2;
    vector1 c;
    int i2_min,i2_max;

    for(i1 = 0;i1 < celllist->nx;i1++)
    {
        cell_index1 = i1;

        for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
        {
            i2_i = INDEX(i2,celllist->nx);
            cell_index2 = i2_i;
            c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);

            if(cell_index1 == cell_index2)
            {
                accumulate_site_energy1_in_same_cell(r,celllist,cell_index1,
                                  e,xi);
            }
            else if(cell_index1 < cell_index2)
            {
                accumulate_site_energy1_in_different_cells(r,celllist,cell_index1,
                                         cell_index2,c,
                                         e,xi);
            }
        }
    }

}

/*---------------------------------------------------------------------------*/
void accumulate_site_energy1_in_same_cell(vector1 *r,linked_cell_list *celllist,int k,
                        double *e,double xi)
{
    int i,j;
    double dr2;
    vector1 dr;
    

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];

                dr2 = SQR(dr[0]);
                if(dr2 < celllist->rc2)
                {
                        *e += xi * exp(-0.5 * 1 * dr2);
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_energy1_in_different_cells(vector1 *r,linked_cell_list *celllist,
                              int k1,int k2,vector1 c,
                              double *e,double xi)
{
    int i,j;
    double dr2;
    vector1 dr;
    

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];

            dr2 = SQR(dr[0]);
            if(dr2 < celllist->rc2)
            {
                    *e += xi * exp(-0.5 * 1 * dr2);
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void accumulate_site_energy2(vector2 *r,linked_cell_list *celllist,box_geometry *geometry,
           double *e,double xi)
{
    int i1,j1,i2,j2,i2_i,j2_i;
    int cell_index1,cell_index2;
    vector2 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_energy2_in_same_cell(r,celllist,cell_index1,
                                               e,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_energy2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     e,xi);
                        }
                    }
                }
                

            }
        }
        
    }
    else
    {
            /*  box deformation  */
        

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_energy2_in_same_cell(r,celllist,cell_index1,
                                               e,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_energy2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     e,xi);
                        }
                    }
                }
                
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = j1 * celllist->nx + i1;

                
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = j2_i * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_energy2_in_same_cell(r,celllist,cell_index1,
                                               e,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_energy2_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     e,xi);
                        }
                    }
                }
                

            }
        }
        
    }

}

/*---------------------------------------------------------------------------*/
void accumulate_site_energy2_in_same_cell(vector2 *r,linked_cell_list *celllist,int k,
                        double *e,double xi)
{
    int i,j;
    double dr2;
    vector2 dr;
    

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

                dr2 = SQR(dr[0]) + SQR(dr[1]);
                if(dr2 < celllist->rc2)
                {
                        *e += xi * exp(-0.5 * 2 * dr2);
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_energy2_in_different_cells(vector2 *r,linked_cell_list *celllist,
                              int k1,int k2,vector2 c,
                              double *e,double xi)
{
    int i,j;
    double dr2;
    vector2 dr;
    

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];

            dr2 = SQR(dr[0]) + SQR(dr[1]);
            if(dr2 < celllist->rc2)
            {
                    *e += xi * exp(-0.5 * 2 * dr2);
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
void accumulate_site_energy3(vector3 *r,linked_cell_list *celllist,box_geometry *geometry,
           double *e,double xi)
{
    int i1,j1,k1,i2,j2,k2,i2_i,j2_i,k2_i;
    int cell_index1,cell_index2;
    vector3 c;
    int i2_min,i2_max;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_energy3_in_same_cell(r,celllist,cell_index1,
                                               e,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_energy3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     e,xi);
                        }
                    }
                }
                }

            }
        }
        }
    }
    else
    {
            /*  box deformation  */
        for(k1 = 0;k1 < celllist->nz;k1++)
               {

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_energy3_in_same_cell(r,celllist,cell_index1,
                                               e,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_energy3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     e,xi);
                        }
                    }
                }
                }
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;

                for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);


                        if(cell_index1 == cell_index2)
                        {
                            accumulate_site_energy3_in_same_cell(r,celllist,cell_index1,
                                               e,xi);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            accumulate_site_energy3_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     e,xi);
                        }
                    }
                }
                }

            }
        }
        }
    }

}

/*---------------------------------------------------------------------------*/
void accumulate_site_energy3_in_same_cell(vector3 *r,linked_cell_list *celllist,int k,
                        double *e,double xi)
{
    int i,j;
    double dr2;
    vector3 dr;
    

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

                dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
                if(dr2 < celllist->rc2)
                {
                        *e += xi * exp(-0.5 * 3 * dr2);
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_site_energy3_in_different_cells(vector3 *r,linked_cell_list *celllist,
                              int k1,int k2,vector3 c,
                              double *e,double xi)
{
    int i,j;
    double dr2;
    vector3 dr;
    

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            dr[0] = r[i][0] - r[j][0] + c[0];
dr[1] = r[i][1] - r[j][1] + c[1];
dr[2] = r[i][2] - r[j][2] + c[2];

            dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);
            if(dr2 < celllist->rc2)
            {
                    *e += xi * exp(-0.5 * 3 * dr2);
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/


void accumulate_bond_force1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           vector1 *f)
{
    int i,j,k;
    vector1 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }

                f[i][0] -= 1 * dr[0];
              f[j][0] += 1 * dr[0];
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

                f[i][0] -= 1 * dr[0];
              f[j][0] += 1 * dr[0];
        }
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_force2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           vector2 *f)
{
    int i,j,k;
    vector2 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }

                f[i][0] -= 2 * dr[0];
              f[j][0] += 2 * dr[0];
f[i][1] -= 2 * dr[1];
              f[j][1] += 2 * dr[1];
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

                f[i][0] -= 2 * dr[0];
              f[j][0] += 2 * dr[0];
f[i][1] -= 2 * dr[1];
              f[j][1] += 2 * dr[1];
        }
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_force3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           vector3 *f)
{
    int i,j,k;
    vector3 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }
if(dr[2] >= 0.5 * geometry->l[2])
                      {
                          dr[2] -= geometry->l[2];
                      }
                      else if(dr[2] < - 0.5 * geometry->l[2])
                      {
                          dr[2] += geometry->l[2];
                      }

                f[i][0] -= 3 * dr[0];
              f[j][0] += 3 * dr[0];
f[i][1] -= 3 * dr[1];
              f[j][1] += 3 * dr[1];
f[i][2] -= 3 * dr[2];
              f[j][2] += 3 * dr[2];
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            if(dr[2] >= 0.5 * geometry->l[2])
                   {
                       dr[2] -= geometry->l[2];
                   }
                   else if(dr[2] < - 0.5 * geometry->l[2])
                   {
                       dr[2] += geometry->l[2];
                   }

                f[i][0] -= 3 * dr[0];
              f[j][0] += 3 * dr[0];
f[i][1] -= 3 * dr[1];
              f[j][1] += 3 * dr[1];
f[i][2] -= 3 * dr[2];
              f[j][2] += 3 * dr[2];
        }
    }
}

void accumulate_bond_stress1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           symmetric_tensor1 sigma)
{
    int i,j,k;
    vector1 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }

                sigma[0] += 1 * dr[0] * dr[0];
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

                sigma[0] += 1 * dr[0] * dr[0];
        }
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_stress2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           symmetric_tensor2 sigma)
{
    int i,j,k;
    vector2 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }

                sigma[0] += 2 * dr[0] * dr[0];
sigma[1] += 2 * dr[1] * dr[1];
sigma[2] += 2 * dr[0] * dr[1];
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

                sigma[0] += 2 * dr[0] * dr[0];
sigma[1] += 2 * dr[1] * dr[1];
sigma[2] += 2 * dr[0] * dr[1];
        }
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_stress3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           symmetric_tensor3 sigma)
{
    int i,j,k;
    vector3 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }
if(dr[2] >= 0.5 * geometry->l[2])
                      {
                          dr[2] -= geometry->l[2];
                      }
                      else if(dr[2] < - 0.5 * geometry->l[2])
                      {
                          dr[2] += geometry->l[2];
                      }

                sigma[0] += 3 * dr[0] * dr[0];
sigma[1] += 3 * dr[1] * dr[1];
sigma[2] += 3 * dr[2] * dr[2];
sigma[3] += 3 * dr[0] * dr[1];
sigma[4] += 3 * dr[1] * dr[2];
sigma[5] += 3 * dr[2] * dr[0];
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            if(dr[2] >= 0.5 * geometry->l[2])
                   {
                       dr[2] -= geometry->l[2];
                   }
                   else if(dr[2] < - 0.5 * geometry->l[2])
                   {
                       dr[2] += geometry->l[2];
                   }

                sigma[0] += 3 * dr[0] * dr[0];
sigma[1] += 3 * dr[1] * dr[1];
sigma[2] += 3 * dr[2] * dr[2];
sigma[3] += 3 * dr[0] * dr[1];
sigma[4] += 3 * dr[1] * dr[2];
sigma[5] += 3 * dr[2] * dr[0];
        }
    }
}

void accumulate_bond_energy1(vector1 *r,transient_bonds *bonds,box_geometry *geometry,
           double *e)
{
    int i,j,k;
    vector1 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }

                *e += 0.5 * 1 * SQR(dr[0]);
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

                *e += 0.5 * 1 * SQR(dr[0]);
        }
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_energy2(vector2 *r,transient_bonds *bonds,box_geometry *geometry,
           double *e)
{
    int i,j,k;
    vector2 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }

                *e += 0.5 * 2 * SQR(dr[0]);
*e += 0.5 * 2 * SQR(dr[1]);
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            

                *e += 0.5 * 2 * SQR(dr[0]);
*e += 0.5 * 2 * SQR(dr[1]);
        }
    }
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_energy3(vector3 *r,transient_bonds *bonds,box_geometry *geometry,
           double *e)
{
    int i,j,k;
    vector3 dr;
    

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[0] >= 0.5 * geometry->l[0])
                      {
                          dr[0] -= geometry->l[0];
                      }
                      else if(dr[0] < - 0.5 * geometry->l[0])
                      {
                          dr[0] += geometry->l[0];
                      }
if(dr[1] >= 0.5 * geometry->l[1])
                      {
                          dr[1] -= geometry->l[1];
                      }
                      else if(dr[1] < - 0.5 * geometry->l[1])
                      {
                          dr[1] += geometry->l[1];
                      }
if(dr[2] >= 0.5 * geometry->l[2])
                      {
                          dr[2] -= geometry->l[2];
                      }
                      else if(dr[2] < - 0.5 * geometry->l[2])
                      {
                          dr[2] += geometry->l[2];
                      }

                *e += 0.5 * 3 * SQR(dr[0]);
*e += 0.5 * 3 * SQR(dr[1]);
*e += 0.5 * 3 * SQR(dr[2]);
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            dr[0] = r[i][0] - r[j][0];
dr[1] = r[i][1] - r[j][1];
dr[2] = r[i][2] - r[j][2];

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            if(dr[2] >= 0.5 * geometry->l[2])
                   {
                       dr[2] -= geometry->l[2];
                   }
                   else if(dr[2] < - 0.5 * geometry->l[2])
                   {
                       dr[2] += geometry->l[2];
                   }

                *e += 0.5 * 3 * SQR(dr[0]);
*e += 0.5 * 3 * SQR(dr[1]);
*e += 0.5 * 3 * SQR(dr[2]);
        }
    }
}

/*---------------------------------------------------------------------------*/
