dnl ---------------------------------------------------------------------------
dnl
dnl  force.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007,2008,2018 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')dnl
include(`scan.m4')dnl
GENERATE_HEADER(force.c,
                calculate the random and interaction forces.)

#include "common.h"
#include "force.h"

GENERATE_HEADER_FOR_SCAN

/*---------------------------------------------------------------------------*/
PROTOTYPE_FUNCTIONS(`dnl
static void generate_random_force@D@(bonded_particles@D@ *particles,rng_state *rng);
static void clear_interaction_force@D@(bonded_particles@D@ *particles);')

static void accumulate_bond_stress(bonded_particles *particles,
                                   box_geometry *geometry);
static void accumulate_site_stress(bonded_particles *particles,
                                   linked_cell_list *celllist,
                                   box_geometry *geometry,double xi);
static void accumulate_bond_energy(bonded_particles *particles,
                                   box_geometry *geometry);
static void accumulate_site_energy(bonded_particles *particles,
                                   linked_cell_list *celllist,
                                   box_geometry *geometry,double xi);

PROTOTYPE_FOR_ALL_BONDS(dnl
`accumulate_bond_force',dnl
`vector@D@ *f')
PROTOTYPE_FOR_ALL_BONDS(dnl
`accumulate_bond_stress',dnl
`symmetric_tensor@D@ sigma')
PROTOTYPE_FOR_ALL_BONDS(dnl
`accumulate_bond_energy',dnl
`double *e')

PROTOTYPE_FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`accumulate_site_force',dnl
`vector@D@ *f,double xi')
PROTOTYPE_FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`accumulate_site_stress',dnl
`symmetric_tensor@D@ sigma,double xi')
PROTOTYPE_FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`accumulate_site_energy',dnl
`double *e,double xi')

/*---------------------------------------------------------------------------*/
void generate_random_force(bonded_particles *particles,rng_state *rng)
{
    SWITCH_FUNCTION(particles->dimension,
                    generate_random_force@D@(&(particles->particles@D@),rng);)
}

/*---------------------------------------------------------------------------*/
void clear_interaction_force(bonded_particles *particles)
{
    SWITCH_FUNCTION(particles->dimension,
                    clear_interaction_force@D@(&(particles->particles@D@));)
}

/*---------------------------------------------------------------------------*/
void clear_stress_tensor(bonded_particles *particles)
{
    SWITCH_FUNCTION(particles->dimension,
                    SYMMETRIC_TENSOR@D@(particles->particles@D@.sigma[@@] = 0.0;))
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_force(bonded_particles *particles,
                           box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    accumulate_bond_force@D@(particles->particles@D@.r,
                    &(particles->bonds),geometry,
                    particles->particles@D@.f);)
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_force_star(bonded_particles *particles,
                                box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    accumulate_bond_force@D@(particles->particles@D@.r_star,
                    &(particles->bonds),geometry,
                    particles->particles@D@.f_star);)
}

/*---------------------------------------------------------------------------*/
void accumulate_site_force(bonded_particles *particles,
                           linked_cell_list *celllist,
                           box_geometry *geometry,double xi)
{
    SWITCH_FUNCTION(particles->dimension,
                    accumulate_site_force@D@(particles->particles@D@.r,
                                             celllist,geometry,
                                             particles->particles@D@.f,xi);)
}

/*---------------------------------------------------------------------------*/
void accumulate_site_force_star(bonded_particles *particles,
                                linked_cell_list *celllist,
                                box_geometry *geometry,double xi)
{
    SWITCH_FUNCTION(particles->dimension,
                    accumulate_site_force@D@(particles->particles@D@.r_star,
                                             celllist,geometry,
                                             particles->particles@D@.f_star,xi);)
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_stress(bonded_particles *particles,
                            box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    accumulate_bond_stress@D@(particles->particles@D@.r,
                    &(particles->bonds),geometry,
                    particles->particles@D@.sigma);)
}

/*---------------------------------------------------------------------------*/
void accumulate_site_stress(bonded_particles *particles,
                            linked_cell_list *celllist,
                            box_geometry *geometry,double xi)
{
    SWITCH_FUNCTION(particles->dimension,
                    accumulate_site_stress@D@(particles->particles@D@.r,
                                              celllist,geometry,
                                              particles->particles@D@.sigma,xi);)
}

/*---------------------------------------------------------------------------*/
void accumulate_bond_energy(bonded_particles *particles,
                            box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    accumulate_bond_energy@D@(particles->particles@D@.r,
                    &(particles->bonds),geometry,
                    &(particles->particles@D@.e[0]));)
}

/*---------------------------------------------------------------------------*/
void accumulate_site_energy(bonded_particles *particles,
                            linked_cell_list *celllist,
                            box_geometry *geometry,double xi)
{
    SWITCH_FUNCTION(particles->dimension,
                    accumulate_site_energy@D@(particles->particles@D@.r,
                                              celllist,geometry,
                                              &(particles->particles@D@.e[1]),xi);)
}

/*---------------------------------------------------------------------------*/
void calculate_interaction_force_without_noise(bonded_particles *particles,
                                               linked_cell_list *celllist,
                                               box_geometry *geometry,
                                               double xi)
{
    clear_interaction_force(particles);

    construct_linked_cell_list(celllist,particles,geometry);

    accumulate_site_force(particles,celllist,geometry,xi);
    accumulate_bond_force(particles,geometry);
}

/*---------------------------------------------------------------------------*/
void calculate_stress_tensor(bonded_particles *particles,
                             linked_cell_list *celllist,
                             box_geometry *geometry,double xi)
{
    clear_stress_tensor(particles);

    construct_linked_cell_list(celllist,particles,geometry);

    accumulate_site_stress(particles,celllist,geometry,xi);
    accumulate_bond_stress(particles,geometry);
}

/*---------------------------------------------------------------------------*/
void calculate_potential_energy(bonded_particles *particles,
                                linked_cell_list *celllist,
                                box_geometry *geometry,double xi)
{
    SWITCH_FUNCTION(particles->dimension,
                    particles->particles@D@.e[0] = 0.0;
                    particles->particles@D@.e[1] = 0.0;)

    construct_linked_cell_list(celllist,particles,geometry);

    accumulate_site_energy(particles,celllist,geometry,xi);
    accumulate_bond_energy(particles,geometry);
}

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
void generate_random_force@D@(bonded_particles@D@ *particles,rng_state *rng)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        VECTOR@D@(particles->w[i][@@] = nrand(rng);)
    }
}

/*---------------------------------------------------------------------------*/
void clear_interaction_force@D@(bonded_particles@D@ *particles)
{
    int i;

    for(i = 0;i < particles->n;i++)
    {
        VECTOR@D@(particles->f[i][@@] = 0.0;)
        VECTOR@D@(particles->f_star[i][@@] = 0.0;)
    }
}')

/*---------------------------------------------------------------------------*/
FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`accumulate_site_force',dnl
`vector@D@ *f,double xi',dnl
`f,xi',dnl
`',dnl
`dnl
    VECTOR@D@(f[i][@@] += xi * exp(-0.5 * @D@ * dr2) * @D@ * dr[@@];
              f[j][@@] -= xi * exp(-0.5 * @D@ * dr2) * @D@ * dr[@@];)')

FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`accumulate_site_stress',dnl
`symmetric_tensor@D@ sigma,double xi',dnl
`sigma,xi',dnl
`',dnl
`dnl
    SYMMETRIC_TENSOR@D@(sigma[@@] -= xi * exp(-0.5 * @D@ * dr2) * @D@ * dr[@1@] * dr[@2@];)')

FOR_ALL_PAIRS_WITH_CELL_LIST(dnl
`accumulate_site_energy',dnl
`double *e,double xi',dnl
`e,xi',dnl
`',dnl
`dnl
    *e += xi * exp(-0.5 * @D@ * dr2);')

FOR_ALL_BONDS(dnl
`accumulate_bond_force',dnl
`vector@D@ *f',dnl
`',dnl
`dnl
    VECTOR@D@(f[i][@@] -= @D@ * dr[@@];
              f[j][@@] += @D@ * dr[@@];)')

FOR_ALL_BONDS(dnl
`accumulate_bond_stress',dnl
`symmetric_tensor@D@ sigma',
`',dnl
`dnl
    SYMMETRIC_TENSOR@D@(sigma[@@] += @D@ * dr[@1@] * dr[@2@];)')

FOR_ALL_BONDS(dnl
`accumulate_bond_energy',dnl
`double *e',
`',dnl
`dnl
    VECTOR@D@(*e += 0.5 * @D@ * SQR(dr[@@]);)')

/*---------------------------------------------------------------------------*/
