/*---------------------------------------------------------------------------*/

/*!  \file  force.h
     \brief  calculate the random and interaction forces.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_FORCE_H
#define INCLUDE_FORCE_H

#include "vector.h"
#include "particles.h"
#include "celllist.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  generate random forces acting on bonded particles.
     \param  particles  bonded particles  */
void generate_random_force(bonded_particles *particles,rng_state *rng);
/*!  \brief  cleaer interaction forces acting on bonded particles.
     \param  particles  bonded particles  */
void clear_interaction_force(bonded_particles *particles);
/*!  \brief  accumulate the tethering interaction forces acting on bonded particles.
     \param  particles  bonded particles
     \param  geometry  box geometry  */
void accumulate_bond_force(bonded_particles *particles,
                           box_geometry *geometry);
void accumulate_bond_force_star(bonded_particles *particles,
                                box_geometry *geometry);
/*!  \brief  accumulate the site interaction forces acting on bonded particles.
     \param  particles  bonded particles
     \param  celllist  linked cell list
     \param  geometry  box geometry
     \param  xi  bond fugacity  */
void accumulate_site_force(bonded_particles *particles,
                           linked_cell_list *celllist,
                           box_geometry *geometry,double xi);
void accumulate_site_force_star(bonded_particles *particles,
                                linked_cell_list *celllist,
                                box_geometry *geometry,double xi);
/*!  \brief  calculate the tethering and site interaction forces (without noise) acting on bonded particles.
     \param  particles  bonded particles
     \param  celllist  linked cell list
     \param  geometry  box geometry
     \param  xi  bond fugacity  */
void calculate_interaction_force_without_noise(bonded_particles *particles,
                                               linked_cell_list *celllist,
                                               box_geometry *geometry,
                                               double xi);
/*!  \brief  calculate the stress tensor.
     \param  particles  bonded particles
     \param  celllist  linked cell list
     \param  geometry  box geometry
     \param  xi  bond fugacity  */
void calculate_stress_tensor(bonded_particles *particles,
                             linked_cell_list *celllist,
                             box_geometry *geometry,double xi);

/*!  \brief  calculate the potential energy.
     \param  particles  bonded particles
     \param  celllist  linked cell list
     \param  geometry  box geometry
     \param  xi  bond fugacity  */
void calculate_potential_energy(bonded_particles *particles,
                                linked_cell_list *celllist,
                                box_geometry *geometry,double xi);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
