/*---------------------------------------------------------------------------*/

/*!  \file  geometry.h
     \brief  the geometry infomation for the box.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_GEOMETRY_H
#define INCLUDE_GEOMETRY_H

#include <stdio.h>
#include "condition.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  geometry for a box  */
typedef struct box_geometry
{
        /*!  \brief  dimension of a box.  */
    int dimension;
        /*!  \brief  lengthes of edges along to x-, y-, and z-axis.  */
    double l[3];

        /*!  \brief  phase of shear.  */
    double theta;
        /*!  \brief  gap between cells by shear deformation.  */
    double delta_l;
        /*!  \brief  the displacement index  */
    int c;
} box_geometry;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new box geometry.  */
box_geometry *allocate_box_geometry(void);
/*!  \brief  free a box geometry.
     \param  geometry  box geometry to free  */
void free_box_geometry(box_geometry *geometry);

/*!  \brief  calculate the volume of a box.
     \param  geometry  geometry of a box  */
double calculate_volume_of_box(box_geometry *geometry);

/*!  \brief  print the current geometry to \a fp.
     \param  fp  file pointer
     \param  geometry  geometry of a box  */
void print_geometry(FILE *fp,box_geometry *geometry);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
