/*---------------------------------------------------------------------------*/

/*!  \file  integrator.h
     \brief  integrate the Langevin equation numerically.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_INTEGRATOR_H
#define INCLUDE_INTEGRATOR_H

#include "condition.h"
#include "particles.h"
#include "celllist.h"
#include "geometry.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  integrate the Langevin equation for bonded particles.
     \param  particles  bonded particles
     \param  celllist  linked cell list
     \param  geometry  box geometry
     \param  rng  random number generator
     \param  xi  bond fugacity
     \param  dt  size of the time step  */
void integrate_langevin_equation(simulation_condition *condition,
                                 bonded_particles *particles,
                                 linked_cell_list *celllist,
                                 box_geometry *geometry,rng_state *rng);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
