/*---------------------------------------------------------------------------*/

/*!  \file  load.h
     \brief  load the state from a HDF5 file  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2009,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_LOAD_H
#define INCLUDE_LOAD_H

/*---------------------------------------------------------------------------*/
#include "rng.h"
#include "geometry.h"
#include "particles.h"
#include "condition.h"

/*---------------------------------------------------------------------------*/
/*!  \brief load the state of the system from the hdf5 file.
     \param  h5filename  hdf5 file name
     \param  condition  simulation condition
     \param  geometry  geometry of the box
     \param  particles  bonded particles
     \param  rng_state  random number generator state */
void h5_load_state_from_file(char *h5filename,
                             simulation_condition *condition,
                             box_geometry *geometry,
                             bonded_particles **particles,
                             rng_state **rng);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
