dnl ---------------------------------------------------------------------------
dnl
dnl  luautil.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2005-2007 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  process this file by using m4 to produce luautil.c
dnl    m4 < luautil.c.m4 > luautil.c
dnl
dnl ---------------------------------------------------------------------------
dnl  VALID_VARIABLE
dnl  $1  type for check of the validity
dnl  $2  lua_State
define(`VALID_VARIABLE',`ifelse($1,boolean,(lua_isboolean($2,-1) || lua_isnumber($2,-1)),lua_is$1($2,-1))')dnl
dnl ---------------------------------------------------------------------------
dnl  TO_VARIABLE
dnl  $1  type for conversion
dnl  $2  lua_State
define(`TO_VARIABLE',`ifelse($1,boolean,(lua_isboolean($2,-1) ? (lua_toboolean($2,-1)) : (lua_tonumber($2,-1))),lua_to$1($2,-1))')dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET
dnl  $1  type for function name
dnl  $2  actual type in C
dnl  $3  type in Lua
define(`LUAU_GET',`dnl
int luau_get_$1(lua_State *L,const char *name,$2 *value)
{
    luau_get_variable(L,name);

    if(!VALID_VARIABLE($3,L))
    {
        fprintf(stderr,"%s is not a valid $3\n",name);
        return -1;
    }

    *value = ($2)TO_VARIABLE($3,L);

    lua_pop(L,1);

    return 0;
}dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_OR_EXIT
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_GET_OR_EXIT',`dnl
int luau_get_$1_or_exit(lua_State *L,const char *name,$2 *value)
{
    int i;

    if((i = luau_get_$1(L,name,value)) < 0)
    {
        exit(EXIT_FAILURE);
    }

    return i;
}dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_ARRAY
dnl  $1  type for function name
dnl  $2  actual type in C
dnl  $3  type in Lua
define(`LUAU_GET_ARRAY',`dnl
int luau_get_$1_array(lua_State *L,const char *name,$2 **value)
{
    int i,n;

    luau_get_variable(L,name);

    if(!lua_istable(L,-1))
    {
        fprintf(stderr,"%s is not a valid table\n",name);
        return -1;
    }

    n = luaL_getn(L,-1);
    *value = xmalloc(sizeof($2) * n);

    for(i = 0;i < n;i++)
    {
        lua_pushnumber(L,i + 1);
        lua_rawget(L, -2);

        if(!VALID_VARIABLE($3,L))
        {
            fprintf(stderr,"%s[%d] is not a valid $3\n",name,i);
            return -1;
        }
        (*value)[i] = ($2)TO_VARIABLE($3,L);

        lua_pop(L,1);
    }

    lua_pop(L,1);

    return n;
}dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_ARRAY_OR_EXIT
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_GET_ARRAY_OR_EXIT',`dnl
int luau_get_$1_array_or_exit(lua_State *L,const char *name,$2 **value)
{
    int i;

    if((i = luau_get_$1_array(L,name,value)) < 0)
    {
        exit(EXIT_FAILURE);
    }

    return i;
}dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_ARRAY2D
dnl  $1  type for function name
dnl  $2  actual type in C
dnl  $3  type in Lua
define(`LUAU_GET_ARRAY2D',`dnl
int *luau_get_$1_array2d(lua_State *L,const char *name,$2 ***value)
{
    static int n[2];
    int i,j;

    luau_get_variable(L,name);

    if(!lua_istable(L,-1))
    {
        fprintf(stderr,"%s is not a valid table\n",name);
        return NULL;
    }
    n[0] = luaL_getn(L,-1);

    lua_pushnumber(L,1);
    lua_gettable(L,-2);
    if(!lua_istable(L,-1))
    {
        fprintf(stderr,"%s[%d] is not a valid table\n",name,0);
        return NULL;
    }
    n[1] = luaL_getn(L,-1);
    lua_pop(L,1);

    *value = xmalloc(sizeof($2 *) * n[0]);
    **value = xmalloc(sizeof($2) * n[0] * n[1]);
    for(i = 0;i < n[0];i++)
        (*value)[i] = (*value)[0] + i * n[1];


    for(i = 0;i < n[0];i++)
    {
        lua_pushnumber(L,i + 1);
        lua_rawget(L, -2);

        if((!lua_istable(L,-1)) || (n[1] != luaL_getn(L,-1)))
        {
            fprintf(stderr,"%s[%d] is not a valid table\n",name,i);
            return NULL;
        }

        for(j = 0;j < n[1];j++)
        {
            lua_pushnumber(L,j + 1);
            lua_rawget(L, -2);
            if(!VALID_VARIABLE($3,L))
            {
                fprintf(stderr,"%s[%d][%d] is not a valid $3\n",name,i,j);
                return NULL;
            }
            (*value)[i][j] = ($2)TO_VARIABLE($3,L);

            lua_pop(L,1);
        }
        lua_pop(L,1);
    }

    lua_pop(L,1);

    return n;
}dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_ARRAY2D_OR_EXIT
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_GET_ARRAY2D_OR_EXIT',`dnl
int *luau_get_$1_array2d_or_exit(lua_State *L,const char *name,$2 ***value)
{
    int *i;

    if((i = luau_get_$1_array2d(L,name,value)) == NULL)
    {
        exit(EXIT_FAILURE);
    }

    return i;
}dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_FREE_ARRAY
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_FREE_ARRAY',`dnl
void luau_free_$1_array($2 *value)
{
    free(value);
}
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_FREE_ARRAY
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_FREE_ARRAY2D',`dnl
void luau_free_$1_array2d($2 **value)
{
    free(*value);
    free(value);
}
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl
/*---------------------------------------------------------------------------*/

/*!  \file  luautil.c
     \brief  some utility functions for Lua.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005-2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from luautil.c.m4.  */

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#include "xmalloc.h"

#ifdef HAVE_LUA_H
#include <lua.h>
#endif
#ifdef HAVE_LUALIB_H
#include <lualib.h>
#endif
#ifdef HAVE_LAUXLIB_H
#include <lauxlib.h>
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "luautil.h"

#ifdef HAVE_LUA_RAWLEN
#define luaL_getn(l,i) lua_rawlen(l,(i))
#define lua_strlen(l,i) lua_rawlen(l,(i))
#endif

/*---------------------------------------------------------------------------*/
static void luau_get_variable(lua_State *L,const char *name);

/*---------------------------------------------------------------------------*/
void luau_get_variable(lua_State *L,const char *name)
{
    if(lua_isnil(L,-1))
    {
            /*  No table is pushed. This is a global variable.  */
        lua_getglobal(L,name);
    }
    else if(lua_istable(L,-1))
    {
            /*  Member of a table.  */
        lua_pushstring(L,name);
        lua_gettable(L,-2);
    }
}

/*---------------------------------------------------------------------------*/
lua_State *luau_open_file(const char *filename)
{
    lua_State *L;

#ifdef HAVE_LUAL_NEWSTATE
        /*  For Lua 5.1.  */
    L = luaL_newstate();
#else
    L = lua_open();
#endif
#ifdef HAVE_LUAL_OPENLIBS
        /*  For Lua 5.1.  */
    luaL_openlibs(L);
#else
        /*  For Lua 5.0 or previous versions.  */
    luaopen_base(L);
    luaopen_table(L);
    luaopen_io(L);
    luaopen_string(L);
    luaopen_math(L);
#endif

        /*  Some tricks to distinguish global variables.
            Here we push `nil'. If there is `nil' in the stack,
            we treat the variable is global.  */
    lua_pushnil(L);

    if(luaL_loadfile(L,filename) || lua_pcall(L,0,0,0))
    {
        fprintf(stderr,"cannot run the input file.\n");
        fprintf(stderr,"%s\n",lua_tostring(L,-1));
        lua_close(L);

        return NULL;
    }

    return L;
}

/*---------------------------------------------------------------------------*/
int luau_push_table(lua_State *L,const char *name)
{
    lua_getglobal(L,name);

    if(!lua_istable(L,-1))
    {
        fprintf(stderr,"`%s' is not a valid table\n",name);
        return -1;
    }

    return 0;
}

/*---------------------------------------------------------------------------*/
int luau_pop_table(lua_State *L)
{
    if(!lua_istable(L,-1))
    {
        fprintf(stderr,"cannot pop table\n");
        return -1;
    }

    lua_pop(L,1);

    return 0;
}

/*---------------------------------------------------------------------------*/
int luau_push_table_or_exit(lua_State *L,const char *name)
{
    int i;

    if((i = luau_push_table(L,name)) < 0)
    {
        exit(EXIT_FAILURE);
    }

    return i;
}

/*---------------------------------------------------------------------------*/
int luau_pop_table_or_exit(lua_State *L)
{
    int i;

    if((i = luau_pop_table(L)) < 0)
    {
        exit(EXIT_FAILURE);
    }

    return i;
}

/*---------------------------------------------------------------------------*/
LUAU_GET(int,int,number)

/*---------------------------------------------------------------------------*/
LUAU_GET(bool,int,boolean)

/*---------------------------------------------------------------------------*/
LUAU_GET(unsigned_long,unsigned long,number)

/*---------------------------------------------------------------------------*/
LUAU_GET(double,double,number)

/*---------------------------------------------------------------------------*/
int luau_get_string(lua_State *L,const char *name,char **value)
{
    int n;

    luau_get_variable(L,name);

    if(!lua_isstring(L,-1))
    {
        fprintf(stderr,"`%s' is not a valid string\n",name);
        return -1;
    }
    n = lua_strlen(L, -1);
    *value = xmalloc(sizeof(char) * (n + 1));

    strcpy(*value,lua_tostring(L,-1));

    lua_pop(L,1);

    return 0;
}

/*---------------------------------------------------------------------------*/
LUAU_GET_OR_EXIT(int,int)

/*---------------------------------------------------------------------------*/
LUAU_GET_OR_EXIT(bool,int)

/*---------------------------------------------------------------------------*/
LUAU_GET_OR_EXIT(unsigned_long,unsigned long)

/*---------------------------------------------------------------------------*/
LUAU_GET_OR_EXIT(double,double)

/*---------------------------------------------------------------------------*/
int luau_get_string_or_exit(lua_State *L,const char *name,char **value)
{
    int i;

    if((i = luau_get_string(L,name,value)) < 0)
    {
        exit(EXIT_FAILURE);
    }

    return i;
}

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY(int,int,number)

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY(double,double,number)

/*---------------------------------------------------------------------------*/
int luau_get_string_array(lua_State *L,const char *name,char ***value)
{
    int i,n,m;

    luau_get_variable(L,name);

    if(!lua_istable(L,-1))
    {
        fprintf(stderr,"`%s' is not a valid table\n",name);
        return -1;
    }

    n = luaL_getn(L,-1);

    m = 0;
    for(i = 0;i < n;i++)
    {
        lua_pushnumber(L,i + 1);
        lua_rawget(L, -2);

        if(!lua_isstring(L,-1))
        {
            fprintf(stderr,"`%s' is not a valid string\n",name);
            return -1;
        }
        m += lua_strlen(L, -1);

        lua_pop(L,1);
    }

    *value = xmalloc(sizeof(char *) * n);
    **value = xmalloc(sizeof(char) * (m + n));
    for(i = 0;i < n;i++)
    {
        lua_pushnumber(L,i + 1);
        lua_rawget(L, -2);

        m = lua_strlen(L, -1);
        strcpy((*value)[i],lua_tostring(L,-1));

        if(i < n - 1)
            (*value)[i + 1] = (*value)[i] + m + 1;

        lua_pop(L,1);
    }

    lua_pop(L,1);

    return n;
}

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY_OR_EXIT(int,int)

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY_OR_EXIT(double,double)

/*---------------------------------------------------------------------------*/
int luau_get_string_array_or_exit(lua_State *L,const char *name,char ***value)
{
    int i;

    if((i = luau_get_string_array(L,name,value)) < 0)
    {
        exit(EXIT_FAILURE);
    }

    return i;
}

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY2D(int,int,number)

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY2D(bool,int,boolean)

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY2D(double,double,number)

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY2D_OR_EXIT(int,int)

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY2D_OR_EXIT(bool,int)

/*---------------------------------------------------------------------------*/
LUAU_GET_ARRAY2D_OR_EXIT(double,double)

/*---------------------------------------------------------------------------*/
LUAU_FREE_ARRAY(int,int)

/*---------------------------------------------------------------------------*/
LUAU_FREE_ARRAY(double,double)

/*---------------------------------------------------------------------------*/
void luau_free_string(char *value)
{
    free(value);
}

/*---------------------------------------------------------------------------*/
LUAU_FREE_ARRAY2D(int,int)

/*---------------------------------------------------------------------------*/
LUAU_FREE_ARRAY2D(bool,int)

/*---------------------------------------------------------------------------*/
LUAU_FREE_ARRAY2D(double,double)

/*---------------------------------------------------------------------------*/
void luau_free_string_array(char **value)
{
    free(*value);
    free(value);
}

/*---------------------------------------------------------------------------*/
dnl
dnl ---------------------------------------------------------------------------
