dnl ---------------------------------------------------------------------------
dnl
dnl  luautil.h.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2005,2006,2018 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  process this file by using m4 to produce luautil.h.m4
dnl    m4 < luautil.h.m4 > luautil.h
dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_GET',`dnl
/*!  \brief  get $1 value of a variable \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_$1(lua_State *L,const char *name,$2 *value)dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_OR_EXIT
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_GET_OR_EXIT',`dnl
/*!  \brief  get $1 value of a variable \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_$1_or_exit(lua_State *L,const char *name,$2 *value)dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_ARRAY
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_GET_ARRAY',`dnl
/*!  \brief  get $1 value of a variable array \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_$1_array(lua_State *L,const char *name,$2 **value)dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_ARRAY_OR_EXIT
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_GET_ARRAY_OR_EXIT',`dnl
/*!  \brief  get $1 value of a variable array \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_$1_array_or_exit(lua_State *L,const char *name,$2 **value)dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_ARRAY2D
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_GET_ARRAY2D',`dnl
/*!  \brief  get $1 value of a variable 2d-array\a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable 2d-array
     \param  value  value of a variable 2d-array  */
int *luau_get_$1_array2d(lua_State *L,const char *name,$2 ***value)dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_GET_ARRAY2D_OR_EXIT
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_GET_ARRAY2D_OR_EXIT',`dnl
/*!  \brief  get $1 value of a variable 2d-array\a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable 2d-array
     \param  value  value of a variable 2d-array  */
int *luau_get_$1_array2d_or_exit(lua_State *L,const char *name,$2 ***value)dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_FREE_ARRAY
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_FREE_ARRAY',`dnl
/*!  \brief  free $1 array allocated by \a luau_get_$1_array.
     \param  value  $2 array to free  */
void luau_free_$1_array($2 *value)dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl  LUAU_FREE_ARRAY2D
dnl  $1  type for function name
dnl  $2  actual type in C
define(`LUAU_FREE_ARRAY2D',`dnl
/*!  \brief  free $1 2d-array allocated by \a luau_get_$1_array2d.
     \param  value  $2 2d-array to free  */
void luau_free_$1_array2d($2 **value)dnl
'dnl
)dnl
dnl ---------------------------------------------------------------------------
dnl
/*---------------------------------------------------------------------------*/

/*!  \file  luautil.h
     \brief  some utility functions for Lua.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from luautil.h.m4.  */

#ifndef INCLUDE_LUAUTIL_H
#define INCLUDE_LUAUTIL_H

#ifdef HAVE_LUA_H
#include <lua.h>
#endif
#ifdef HAVE_LUALIB_H
#include <lualib.h>
#endif
#ifdef HAVE_LAUXLIB_H
#include <lauxlib.h>
#endif

/*---------------------------------------------------------------------------*/
/*!  \brief  open \a filename as a Lua script.
     \param  lua_State *L  state of Lua
     \param  name  name of a table  */
lua_State *luau_open_file(const char *filename);

/*!  \brief  push \a name into the Lua stack as a table name.
     \param  lua_State *L  state of Lua
     \param  name  name of a table  */
int luau_push_table(lua_State *L,const char *name);
/*!  \brief  pop a table name from the Lua stack.
     \param  lua_State *L  state of Lua  */
int luau_pop_table(lua_State *L);
/*!  \brief  push \a name into the Lua stack as a table name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a table  */
int luau_push_table_or_exit(lua_State *L,const char *name);
/*!  \brief  pop a table name from the Lua stack (exit if an error is occurred).
     \param  lua_State *L  state of Lua  */
int luau_pop_table_or_exit(lua_State *L);

LUAU_GET(int,int);
LUAU_GET(bool,int);
LUAU_GET(unsigned_long,unsigned long);
LUAU_GET(double,double);
/*!  \brief  get string value of a variable \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_string(lua_State *L,const char *name,char **value);
LUAU_GET_OR_EXIT(int,int);
LUAU_GET_OR_EXIT(bool,int);
LUAU_GET_OR_EXIT(unsigned_long,unsigned long);
LUAU_GET_OR_EXIT(double,double);
/*!  \brief  get string value of a variable \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_string_or_exit(lua_State *L,const char *name,char **value);

LUAU_GET_ARRAY(int,int);
LUAU_GET_ARRAY(double,double);
/*!  \brief  get string value of a variable array \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_string_array(lua_State *L,const char *name,char ***value);
LUAU_GET_ARRAY_OR_EXIT(int,int);
LUAU_GET_ARRAY_OR_EXIT(double,double);
/*!  \brief  get string value of a variable array \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_string_array_or_exit(lua_State *L,const char *name,char ***value);

LUAU_GET_ARRAY2D(int,int);
LUAU_GET_ARRAY2D(bool,int);
LUAU_GET_ARRAY2D(double,double);
LUAU_GET_ARRAY2D_OR_EXIT(int,int);
LUAU_GET_ARRAY2D_OR_EXIT(bool,int);
LUAU_GET_ARRAY2D_OR_EXIT(double,double);

/*!  \brief  free string allocated by \a luau_get_string.
     \param  value  string to free  */
void luau_free_string(char *value);

LUAU_FREE_ARRAY(int,int);
LUAU_FREE_ARRAY(double,double);
/*!  \brief  free string array allocated by \a luau_get_string_array.
     \param  value  string array to free  */
void luau_free_string_array(char **value);

LUAU_FREE_ARRAY2D(int,int);
LUAU_FREE_ARRAY2D(bool,int);
LUAU_FREE_ARRAY2D(double,double);

/*---------------------------------------------------------------------------*/
#endif
dnl
dnl ---------------------------------------------------------------------------
