dnl ---------------------------------------------------------------------------
dnl
dnl  scan.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007,2008,2018,2019 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
ifdef(`INCLUDE_SCAN_M4',`',dnl
`dnl
define(`INCLUDE_SCAN_M4')dnl
include(`dimension.m4')`'dnl
define(`GENERATE_HEADER_FOR_SCAN',dnl
`dnl
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "vector.h"
#include "particles.h"
#include "celllist.h"

#define SQR(x) ((x)*(x))

#define INDEX(i,n) (((i) == -1) ? ((n) - 1) : (((i) == (n)) ? (0) : (i)))
#define LATTICE_VECTOR(i,n,c) (((i) == -1) ? (c) : (((i) == (n)) ? (-(c)) : (0)))
#define IS_LOWER_BOUNDARY(i1,i2,n) (((i1) == 0) && ((i2) == -1))
#define IS_UPPER_BOUNDARY(i1,i2,n) (((i1) == ((n) - 1)) && ((i2) == (n)))
')dnl
dnl ---------------------------------------------------------------------------
dnl  PROTOTYPE_FOR_ALL_PAIRS_WITH_CELL_LIST
dnl    perform operations for all the pairs within the threshold.
dnl    the functions for two and three dimensional systems are generated.
dnl  $1 the function name prefix.
dnl  $2 definitions for the additional arguments.
dnl ---------------------------------------------------------------------------
define(`PROTOTYPE_FOR_ALL_PAIRS_WITH_CELL_LIST',`dnl
PROTOTYPE_FUNCTIONS(`dnl
static void $1@D@(vector@D@ *r,linked_cell_list *celllist,box_geometry *geometry,
                  $2);
static void $1@D@_in_same_cell(vector@D@ *r,linked_cell_list *celllist,int k,
                               $2);
static void $1@D@_in_different_cells(vector@D@ *r,linked_cell_list *celllist,
                                     int k1,int k2,vector@D@ c,
                                     $2);')')dnl
dnl ---------------------------------------------------------------------------
dnl  FOR_ALL_PAIRS_WITH_CELL_LIST
dnl    perform operations for all the pairs within the threshold.
dnl    the functions for two and three dimensional systems are generated.
dnl  $1 the function name prefix.
dnl  $2 the definitions for the additional arguments.
dnl  $3 the additional argument list.
dnl  $4 the definitions for the additional local variables.
dnl  $5 the operation to be performed for pairs
dnl     `i', `j' are indices for the pair, and `dr' is the relative position
dnl     (dr = r[i] - r[j]), and `dr2' is the squared distance (dr2 = dr . dr).
dnl ---------------------------------------------------------------------------
define(`FOR_ALL_PAIRS_WITH_CELL_LIST',`dnl
EXPAND_FUNCTIONS(`dnl
void $1@D@(vector@D@ *r,linked_cell_list *celllist,box_geometry *geometry,
           $2)
{
    ifelse(@D@,1,
           `int i1,i2,i2_i;',
           @D@,2,
           `int i1,j1,i2,j2,i2_i,j2_i;'`',
           @D@,3,
           `int i1,j1,k1,i2,j2,k2,i2_i,j2_i,k2_i;'`')
    int cell_index1,cell_index2;
    vector@D@ c;
    int i2_min,i2_max;

    ifelse(@D@,1,
    for(i1 = 0;i1 < celllist->nx;i1++)
    {
        cell_index1 = i1;

        for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
        {
            i2_i = INDEX(i2,celllist->nx);
            cell_index2 = i2_i;
            c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);

            if(cell_index1 == cell_index2)
            {
                $1@D@_in_same_cell(r,celllist,cell_index1,
                                  $3);
            }
            else if(cell_index1 < cell_index2)
            {
                $1@D@_in_different_cells(r,celllist,cell_index1,
                                         cell_index2,c,
                                         $3);
            }
        }
    }
,
    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        ifelse(@D@,3,
               for(k1 = 0;k1 < celllist->nz;k1++)
               {)
        for(j1 = 0;j1 < celllist->ny;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                ifelse(@D@,2,
                       cell_index1 = j1 * celllist->nx + i1;,
                       @D@,3,
                       cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;)

                ifelse(@D@,3,
                       for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);)
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        ifelse(@D@,2,
                               cell_index2 = j2_i * celllist->nx + i2_i;,
                               @D@,3,
                               cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;)

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        ifelse(@D@,3,
                               c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);)


                        if(cell_index1 == cell_index2)
                        {
                            $1@D@_in_same_cell(r,celllist,cell_index1,
                                               $3);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            $1@D@_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     $3);
                        }
                    }
                }
                ifelse(@D@,3,
                       })

            }
        }
        ifelse(@D@,3,
               })
    }
    else
    {
            /*  box deformation  */
        ifelse(@D@,3,
               for(k1 = 0;k1 < celllist->nz;k1++)
               {)

            /*  boundary cells (j = 0, ny - 1)  */
        for(j1 = 0;j1 < celllist->ny;j1 += (celllist->ny - 1))
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                ifelse(@D@,2,
                       cell_index1 = j1 * celllist->nx + i1;,
                       @D@,3,
                       cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;)

                ifelse(@D@,3,
                       for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);)
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 + (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                    {
                        i2_min = i1 + (int)floor(-1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                        i2_max = i1 + (int)ceil(1.0 - (geometry->delta_l * celllist->nx) / geometry->l[0]);
                    }
                    else
                    {
                        i2_min = i1 - 1;
                        i2_max = i1 + 1;
                    }

                    for(i2 = i2_min;i2 <= i2_max;i2++)
                    {
                        i2_i = i2;
                        c[0] = 0.0;

                        while(i2_i < 0)
                        {
                            i2_i += celllist->nx;
                            c[0] += geometry->l[0];
                        }
                        while(i2_i >= celllist->nx)
                        {
                            i2_i -= celllist->nx;
                            c[0] -= geometry->l[0];
                        }

                        if(IS_LOWER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] += geometry->delta_l;
                        }
                        else if(IS_UPPER_BOUNDARY(j1,j2,celllist->ny))
                        {
                            c[0] -= geometry->delta_l;
                        }


                        ifelse(@D@,2,
                               cell_index2 = j2_i * celllist->nx + i2_i;,
                               @D@,3,
                               cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;)

                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        ifelse(@D@,3,
                               c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);)


                        if(cell_index1 == cell_index2)
                        {
                            $1@D@_in_same_cell(r,celllist,cell_index1,
                                               $3);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            $1@D@_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     $3);
                        }
                    }
                }
                ifelse(@D@,3,
                       })
            }
        }


            /*  non-boundary cells  */
        for(j1 = 1;j1 < celllist->ny - 1;j1++)
        {
            for(i1 = 0;i1 < celllist->nx;i1++)
            {
                ifelse(@D@,2,
                       cell_index1 = j1 * celllist->nx + i1;,
                       @D@,3,
                       cell_index1 = (k1 * celllist->ny + j1) * celllist->nx + i1;)

                ifelse(@D@,3,
                       for(k2 = k1 - 1;k2 <= k1 + 1;k2++)
                       {
                           k2_i = INDEX(k2,celllist->nz);)
                for(j2 = j1 - 1;j2 <= j1 + 1;j2++)
                {
                    j2_i = INDEX(j2,celllist->ny);

                    for(i2 = i1 - 1;i2 <= i1 + 1;i2++)
                    {
                        i2_i = INDEX(i2,celllist->nx);

                        ifelse(@D@,2,
                               cell_index2 = j2_i * celllist->nx + i2_i;,
                               @D@,3,
                               cell_index2 = (k2_i * celllist->ny + j2_i) * celllist->nx + i2_i;)

                        c[0] = LATTICE_VECTOR(i2,celllist->nx,geometry->l[0]);
                        c[1] = LATTICE_VECTOR(j2,celllist->ny,geometry->l[1]);
                        ifelse(@D@,3,
                               c[2] = LATTICE_VECTOR(k2,celllist->nz,geometry->l[2]);)


                        if(cell_index1 == cell_index2)
                        {
                            $1@D@_in_same_cell(r,celllist,cell_index1,
                                               $3);
                        }
                        else if(cell_index1 < cell_index2)
                        {
                            $1@D@_in_different_cells(r,celllist,cell_index1,
                                                     cell_index2,c,
                                                     $3);
                        }
                    }
                }
                ifelse(@D@,3,
                       })

            }
        }
        ifelse(@D@,3,
               })
    }
)
}

/*---------------------------------------------------------------------------*/
void $1@D@_in_same_cell(vector@D@ *r,linked_cell_list *celllist,int k,
                        $2)
{
    int i,j;
    double dr2;
    vector@D@ dr;
    $4

    i = celllist->head[k];

    while(!(i < 0))
    {
        j = celllist->head[k];
        while(!(j < 0))
        {
            if(i < j)
            {
                VECTOR@D@(dr[@@] = r[i][@@] - r[j][@@];)

                ifelse(@D@,1,
                       dr2 = SQR(dr[0]);,
                       @D@,2,
                       dr2 = SQR(dr[0]) + SQR(dr[1]);,
                       @D@,3,
                       dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);)
                if(dr2 < celllist->rc2)
                {
                    $5
                }
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
void $1@D@_in_different_cells(vector@D@ *r,linked_cell_list *celllist,
                              int k1,int k2,vector@D@ c,
                              $2)
{
    int i,j;
    double dr2;
    vector@D@ dr;
    $4

    i = celllist->head[k1];
    while(!(i < 0))
    {
        j = celllist->head[k2];
        while(!(j < 0))
        {
            VECTOR@D@(dr[@@] = r[i][@@] - r[j][@@] + c[@@];)

            ifelse(@D@,1,
                   dr2 = SQR(dr[0]);,
                   @D@,2,
                   dr2 = SQR(dr[0]) + SQR(dr[1]);,
                   @D@,3,
                   dr2 = SQR(dr[0]) + SQR(dr[1]) + SQR(dr[2]);)
            if(dr2 < celllist->rc2)
            {
                $5
            }

            j = celllist->list[j];
        }

        i = celllist->list[i];
    }
}

/*---------------------------------------------------------------------------*/
')')dnl
dnl
dnl ---------------------------------------------------------------------------
dnl  PROTOTYPE_FOR_ALL_BONDS
dnl    perform operations for all the active transient bonds.
dnl    the functions for two and three dimensional systems are generated.
dnl  $1 the function name prefix.
dnl  $2 the definitions for the additional arguments.
dnl ---------------------------------------------------------------------------
define(`PROTOTYPE_FOR_ALL_BONDS',`dnl
PROTOTYPE_FUNCTIONS(`dnl
static void $1@D@(vector@D@ *r,transient_bonds *bonds,box_geometry *geometry,
           $2);')')dnl
dnl ---------------------------------------------------------------------------
dnl  FOR_ALL_BONDS
dnl    perform operations for all the active transient bonds.
dnl    the functions for two and three dimensional systems are generated.
dnl  $1 the function name prefix.
dnl  $2 the definitions for the additional arguments.
dnl  $3 the definitions for the additional local variables.
dnl  $4 the operation to be performed for pairs
dnl     `k' is the index for the active bond,
dnl     `i', `j' are indices for the pair, and `dr' is the relative position
dnl     (dr = r[i] - r[j]), and `dr2' is the squared distance (dr2 = dr . dr).
dnl    generate functions to calculate the force or the stress tensor
dnl ---------------------------------------------------------------------------
define(`FOR_ALL_BONDS',`dnl
EXPAND_FUNCTIONS(`dnl
void $1@D@(vector@D@ *r,transient_bonds *bonds,box_geometry *geometry,
           $2)
{
    int i,j,k;
    vector@D@ dr;
    $3

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            VECTOR@D@(dr[@@] = r[i][@@] - r[j][@@];)

            VECTOR@D@(if(dr[@@] >= 0.5 * geometry->l[@@])
                      {
                          dr[@@] -= geometry->l[@@];
                      }
                      else if(dr[@@] < - 0.5 * geometry->l[@@])
                      {
                          dr[@@] += geometry->l[@@];
                      })

            $4
        }
    }
    else
    {
            /*  box deformation  */
        for(k = 0;k < bonds->m_max;k++)
        {
            if(IS_INACTIVE_BOND(bonds->s[k]))
                continue;

            i = bonds->s[k][0];
            j = bonds->s[k][1];

            /*  calculate the bond force.  */
            VECTOR@D@(dr[@@] = r[i][@@] - r[j][@@];)

            if(dr[1] >= 0.5 * geometry->l[1])
            {
                dr[1] -= geometry->l[1];
                dr[0] -= geometry->delta_l;
            }
            else if(dr[1] < - 0.5 * geometry->l[1])
            {
                dr[1] += geometry->l[1];
                dr[0] += geometry->delta_l;
            }
            while(dr[0] >= 0.5 * geometry->l[0])
            {
                dr[0] -= geometry->l[0];
            }
            while(dr[0] < - 0.5 * geometry->l[0])
            {
                dr[0] += geometry->l[0];
            }
            ifelse(@D@,3,
                   if(dr[2] >= 0.5 * geometry->l[2])
                   {
                       dr[2] -= geometry->l[2];
                   }
                   else if(dr[2] < - 0.5 * geometry->l[2])
                   {
                       dr[2] += geometry->l[2];
                   })

            $4
        }
    }
}')')dnl
dnl ---------------------------------------------------------------------------
')dnl
dnl
dnl ---------------------------------------------------------------------------
