/*---------------------------------------------------------------------------*/

/*!  \file  simulator.h
     \brief  a simple transient bond model simulator.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_SIMULATOR_H
#define INCLUDE_SIMULATOR_H

#include "condition.h"
#include "files.h"
#include "geometry.h"
#include "particles.h"
#include "celllist.h"

/*--------------------------------------------------------------------------*/
/*!  \brief  perform a transient bond dynamics simulation.
     \param  condition  simulation condition
     \param  files  data files
     \param  geometry  geometry of the box
     \param  particles  bonded particles
     \param  celllist  linked cell list
     \param  rng  random number generator  */
int simulate(simulation_condition *condition,
             data_files *files,
             box_geometry *geometry,
             bonded_particles *particles,
             linked_cell_list *celllist,
             rng_state *rng);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
