/*---------------------------------------------------------------------------*/

/*!  \file  textwrite.c
     \brief  output the field data as ascii text format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018,2019 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from textwrite.c.m4.  */

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "particles.h"
#include "bonds.h"
#include "textwrite.h"

/*---------------------------------------------------------------------------*/
static void write_positions1_text(char *filename,bonded_particles1 *particles);
static void write_stress1_text(char *filename,bonded_particles1 *particles,
                                 box_geometry *geometry,int append);

static void write_positions2_text(char *filename,bonded_particles2 *particles);
static void write_stress2_text(char *filename,bonded_particles2 *particles,
                                 box_geometry *geometry,int append);

static void write_positions3_text(char *filename,bonded_particles3 *particles);
static void write_stress3_text(char *filename,bonded_particles3 *particles,
                                 box_geometry *geometry,int append);

/*---------------------------------------------------------------------------*/
void write_positions_text(char *filename,bonded_particles *particles)
{
        switch(particles->dimension)
    {
        case 1:
            write_positions1_text(filename,&(particles->particles1));
            break;

        case 2:
            write_positions2_text(filename,&(particles->particles2));
            break;

        case 3:
            write_positions3_text(filename,&(particles->particles3));
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void write_bonds_text(char *filename,bonded_particles *particles)
{
    gzFile fp;
    int i;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# %d\n",total_number_of_bonds(&(particles->bonds)));

    for(i = 0;i < particles->bonds.m_max;i++)
    {
        if(IS_ACTIVE_BOND(particles->bonds.s[i]))
        {
            gzprintf(fp,"%d %d\n",particles->bonds.s[i][0],
                     particles->bonds.s[i][1]);
        }
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_stress_text(char *filename,bonded_particles *particles,
                       box_geometry *geometry,int append)
{
        switch(particles->dimension)
    {
        case 1:
            write_stress1_text(filename,&(particles->particles1),
                                           geometry,append);
            break;

        case 2:
            write_stress2_text(filename,&(particles->particles2),
                                           geometry,append);
            break;

        case 3:
            write_stress3_text(filename,&(particles->particles3),
                                           geometry,append);
            break;

        default:
            break;
    }
}


/*---------------------------------------------------------------------------*/
void write_energy_density_text(char *filename,bonded_particles *particles,
                               box_geometry *geometry,int append)
{
    FILE *fp;
    double v;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    if(append)
        fp = fopen(filename,"at");
    else
        fp = fopen(filename,"wt");

    v = calculate_volume_of_box(geometry);

        switch(particles->dimension)
    {
        case 1:
            fprintf(fp,"%f %f\n",particles->particles1.e[0] / v,particles->particles1.e[1] / v);
            break;

        case 2:
            fprintf(fp,"%f %f\n",particles->particles2.e[0] / v,particles->particles2.e[1] / v);
            break;

        case 3:
            fprintf(fp,"%f %f\n",particles->particles3.e[0] / v,particles->particles3.e[1] / v);
            break;

        default:
            break;
    }

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_box_geometry_text(char *filename,box_geometry *geometry,int append)
{
    FILE *fp;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    if(append)
        fp = fopen(filename,"at");
    else
        fp = fopen(filename,"wt");

    switch(geometry->dimension)
    {
        case 1:
            fprintf(fp,"1\n");
            fprintf(fp,"%f\n",geometry->l[0]);
            break;

        case 2:
            fprintf(fp,"2\n");
            fprintf(fp,"%f %f\n",geometry->l[0],geometry->l[1]);
            fprintf(fp,"%g %d\n",geometry->delta_l,geometry->c);
            break;

        case 3:
            fprintf(fp,"3\n");
            fprintf(fp,"%f %f %f\n",geometry->l[0],geometry->l[1],geometry->l[2]);
            fprintf(fp,"%g %d\n",geometry->delta_l,geometry->c);
            break;

        default:
            break;
    }

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_positions1_text(char *filename,bonded_particles1 *particles)
{
    gzFile fp;
    int i;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# 1 %d\n",particles->n);

    for(i = 0;i < particles->n;i++)
    {
        gzprintf(fp,"%f %d\n",
                        particles->r[i][0],
                        particles->c[i][0]);
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_stress1_text(char *filename,bonded_particles1 *particles,
                          box_geometry *geometry,int append)
{
    FILE *fp;
    symmetric_tensor1 sigma;
    double v;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    v = calculate_volume_of_box(geometry);
    sigma[0] = particles->sigma[0] / v;

    if(append)
        fp = fopen(filename,"at");
    else
        fp = fopen(filename,"wt");

    fprintf(fp,"%g\n",sigma[0]);

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_positions2_text(char *filename,bonded_particles2 *particles)
{
    gzFile fp;
    int i;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# 2 %d\n",particles->n);

    for(i = 0;i < particles->n;i++)
    {
        gzprintf(fp,"%f %f %d %d\n",
                        particles->r[i][0],particles->r[i][1],
                        particles->c[i][0],particles->c[i][1]);
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_stress2_text(char *filename,bonded_particles2 *particles,
                          box_geometry *geometry,int append)
{
    FILE *fp;
    symmetric_tensor2 sigma;
    double v;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    v = calculate_volume_of_box(geometry);
    sigma[0] = particles->sigma[0] / v;
sigma[1] = particles->sigma[1] / v;
sigma[2] = particles->sigma[2] / v;

    if(append)
        fp = fopen(filename,"at");
    else
        fp = fopen(filename,"wt");

    fprintf(fp,"%g %g %g\n",sigma[0],sigma[1],sigma[2]);

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_positions3_text(char *filename,bonded_particles3 *particles)
{
    gzFile fp;
    int i;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# 3 %d\n",particles->n);

    for(i = 0;i < particles->n;i++)
    {
        gzprintf(fp,"%f %f %f %d %d %d\n",
                        particles->r[i][0],particles->r[i][1],particles->r[i][2],
                        particles->c[i][0],particles->c[i][1],particles->c[i][2]);
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_stress3_text(char *filename,bonded_particles3 *particles,
                          box_geometry *geometry,int append)
{
    FILE *fp;
    symmetric_tensor3 sigma;
    double v;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    v = calculate_volume_of_box(geometry);
    sigma[0] = particles->sigma[0] / v;
sigma[1] = particles->sigma[1] / v;
sigma[2] = particles->sigma[2] / v;
sigma[3] = particles->sigma[3] / v;
sigma[4] = particles->sigma[4] / v;
sigma[5] = particles->sigma[5] / v;

    if(append)
        fp = fopen(filename,"at");
    else
        fp = fopen(filename,"wt");

    fprintf(fp,"%g %g %g %g %g %g\n",sigma[0],sigma[1],
                   sigma[2],sigma[3],sigma[4],sigma[5]);

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
