dnl ---------------------------------------------------------------------------
dnl
dnl  textwrite.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007,2008.2018,2019,2021 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')`'dnl
GENERATE_HEADER(textwrite.c,
                output the field data as ascii text format.)

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "particles.h"
#include "bonds.h"
#include "textwrite.h"

/*---------------------------------------------------------------------------*/
PROTOTYPE_FUNCTIONS(`dnl
static void write_positions@D@_text(char *filename,bonded_particles@D@ *particles);
static void write_stress@D@_text(char *filename,bonded_particles@D@ *particles,
                                 box_geometry *geometry,int append);'`')

/*---------------------------------------------------------------------------*/
void write_positions_text(char *filename,bonded_particles *particles)
{
    SWITCH_FUNCTION(particles->dimension,
                    write_positions@D@_text(filename,&(particles->particles@D@));)
}

/*---------------------------------------------------------------------------*/
void write_bonds_text(char *filename,bonded_particles *particles)
{
    gzFile fp;
    int i;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# %d\n",total_number_of_bonds(&(particles->bonds)));

    for(i = 0;i < particles->bonds.m_max;i++)
    {
        if(IS_ACTIVE_BOND(particles->bonds.s[i]))
        {
            gzprintf(fp,"%d %d\n",particles->bonds.s[i][0],
                     particles->bonds.s[i][1]);
        }
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_stress_text(char *filename,bonded_particles *particles,
                       box_geometry *geometry,int append)
{
    SWITCH_FUNCTION(particles->dimension,
                    write_stress@D@_text(filename,&(particles->particles@D@),
                                           geometry,append);)
}


/*---------------------------------------------------------------------------*/
void write_energy_density_text(char *filename,bonded_particles *particles,
                               box_geometry *geometry,int append)
{
    FILE *fp;
    double v;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    if(append)
        fp = fopen(filename,"at");
    else
        fp = fopen(filename,"wt");

    v = calculate_volume_of_box(geometry);

    SWITCH_FUNCTION(particles->dimension,
                    fprintf(fp,"%f %f\n",particles->particles@D@.e[0] / v,particles->particles@D@.e[1] / v);)

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_box_geometry_text(char *filename,box_geometry *geometry,int append)
{
    FILE *fp;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    if(append)
        fp = fopen(filename,"at");
    else
        fp = fopen(filename,"wt");

    switch(geometry->dimension)
    {
        case 1:
            fprintf(fp,"1\n");
            fprintf(fp,"%f\n",geometry->l[0]);
            break;

        case 2:
            fprintf(fp,"2\n");
            fprintf(fp,"%f %f\n",geometry->l[0],geometry->l[1]);
            fprintf(fp,"%g %d\n",geometry->delta_l,geometry->c);
            break;

        case 3:
            fprintf(fp,"3\n");
            fprintf(fp,"%f %f %f\n",geometry->l[0],geometry->l[1],geometry->l[2]);
            fprintf(fp,"%g %d\n",geometry->delta_l,geometry->c);
            break;

        default:
            break;
    }

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
void write_positions@D@_text(char *filename,bonded_particles@D@ *particles)
{
    gzFile fp;
    int i;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# @D@ %d\n",particles->n);

    for(i = 0;i < particles->n;i++)
    {
        ifelse(@D@,1,
               gzprintf(fp,"%f %d\n",
                        particles->r[i][0],
                        particles->c[i][0]);,
               @D@,2,
               gzprintf(fp,"%f %f %d %d\n",
                        particles->r[i][0],particles->r[i][1],
                        particles->c[i][0],particles->c[i][1]);,
               @D@,3,
               gzprintf(fp,"%f %f %f %d %d %d\n",
                        particles->r[i][0],particles->r[i][1],particles->r[i][2],
                        particles->c[i][0],particles->c[i][1],particles->c[i][2]);)
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_stress@D@_text(char *filename,bonded_particles@D@ *particles,
                          box_geometry *geometry,int append)
{
    FILE *fp;
    symmetric_tensor@D@ sigma;
    double v;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    v = calculate_volume_of_box(geometry);
    SYMMETRIC_TENSOR@D@(sigma[@@] = particles->sigma[@@] / v;)

    if(append)
        fp = fopen(filename,"at");
    else
        fp = fopen(filename,"wt");

    ifelse(@D@,1,
           fprintf(fp,"%g\n",sigma[0]);,
           @D@,2,
           fprintf(fp,"%g %g %g\n",sigma[0],sigma[1],sigma[2]);,
           @D@,3,
           fprintf(fp,"%g %g %g %g %g %g\n",sigma[0],sigma[1],
                   sigma[2],sigma[3],sigma[4],sigma[5]);)

    fclose(fp);
}'`')

/*---------------------------------------------------------------------------*/
