/*---------------------------------------------------------------------------*/

/*!  \file  textwrite.h
     \brief  output the position data as gzipped ascii text format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_TEXTWRITE_H
#define INCLUDE_TEXTWRITE_H

#include "particles.h"
#include "geometry.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  write positions of \a particles to \a filename.
     \param  filename  file name
     \param  particles  bonded particles  */
void write_positions_text(char *filename,bonded_particles *particles);
/*!  \brief  write bond indices for \a particles to \a filename.
     \param  filename  file name
     \param  bonds  transient bonds  */
void write_bonds_text(char *filename,bonded_particles *particles);
/*!  \brief  write stress tensor to \a filename.
     \param  filename  file name
     \param  particles  bonded particles
     \param  geometry  box geometry
     \param  append  append or rewrite  */
void write_stress_text(char *filename,bonded_particles *particles,
                       box_geometry *geometry,int append);
/*!  \brief  write potential energy per volume to \a filename.
     \param  filename  file name
     \param  particles  bonded particles
     \param  geometry  box geometry
     \param  append  append or rewrite  */
void write_energy_density_text(char *filename,bonded_particles *particles,
                               box_geometry *geometry,int append);
/*!  \brief  write \a geometry of the simulation box to \a filename.
     \param  filename  file name
     \param  geometry  box geometry
     \param  append  append or rewrite  */
void write_box_geometry_text(char *filename,box_geometry *geometry,
                             int append);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
